EMAIL_TEMPLATES = {
    'user_invitation': 'argomi_invitation.html',
    'user_welcome': 'argomi_welcome.html',
    'user_signup': 'argomi_signup.html',
    'user_change_email': 'argomi_change_email.html',
    'forgot_password': 'argomi_change_password.html',
    'default_message': 'argomi_default_message.html'
}


"""
This module contains publisher role to AWS services
"""
import logging


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


class QueuePublisher(object):
    """
    The current implementation uses SQS as the transport mechanism.
    """

    @staticmethod
    def publish_item(transport, message, message_attributes=None):
        """
        This method is used to publish message by injecting
        transport object like StandardSQS object etc.
        """
        return transport.send_message(message,
                                      message_attributes=message_attributes)




import os
import boto3
import logging
import traceback
from typing import List

from boto3.dynamodb.conditions import Key, Attr
from amaasinfra.authorization.token_utils import TokenAttribute, unpack_token

class AMaaSUser(object):

    __cache_clients = {}
    __environment = os.environ.get('AWS_ENV', 'dev').lower()


    @classmethod
    def create_clients(cls):
        if not cls.__cache_clients:
            region = os.environ.get('AWS_DEFAULT_REGION') or 'ap-southeast-1'
            dynamodb = boto3.resource('dynamodb', region_name=region)
            for table in  ['book_permissions', 'relationships']:
                table_name = cls.get_table_name(table)
                cls.__cache_clients[table_name] = dynamodb.Table(table_name)


    @classmethod
    def get_table_name(cls, table):
        return '_'.join([table, cls.__environment])


    def __init__(self, token: str, logger=None) -> None:
        attributes = unpack_token(token)
        user_amid = attributes.get(TokenAttribute.asset_manager_id.value)
        self.asset_manager_id = int(user_amid) if user_amid else None
        self.username = attributes.get(TokenAttribute.username.value)
        self.email = attributes.get(TokenAttribute.email.value)
        self.firstname = attributes.get(TokenAttribute.first_name.value, '')
        self.lastname = attributes.get(TokenAttribute.last_name.value, '')
        # amid 0-9 are reserved for system users
        self.system_user = self.asset_manager_id < 10 if self.asset_manager_id else False
        self.relationships: List[dict] = []
        self.data_sources: List[int] = []
        self.asset_manager_permissions: List[int] = []
        self.book_permissions: List[dict] = []
        self.logger = logger or logging.getLogger()
        self.create_clients()


    def load_relationships(self) -> 'AMaaSUser':
        if self.asset_manager_id is None:
            return self

        results = self._dynamo_query('relationships',
                                     Select='SPECIFIC_ATTRIBUTES',
                                     IndexName='related_id_index',
                                     ProjectionExpression='asset_manager_id,related_id,relationship_type,relationship_status',
                                     KeyConditionExpression=Key('related_id').eq(self.asset_manager_id),
                                     FilterExpression=Attr('relationship_status').eq('Active'))
        self.relationships = results
        return self


    def load_data_sources(self) -> 'AMaaSUser':
        if self.asset_manager_id is None:
            return self

        if not self.relationships:
            self.load_relationships()

        asset_manager_ids = [rel.get('asset_manager_id') for rel in self.relationships]
        data_providers = set()
        for asset_manager_id in asset_manager_ids:
            results = self._dynamo_query('relationships',
                                         Select='SPECIFIC_ATTRIBUTES',
                                         ProjectionExpression='related_id',
                                         KeyConditionExpression=Key('asset_manager_id').eq(asset_manager_id),
                                         FilterExpression=Attr('relationship_status').eq('Active') & Attr('relationship_type').eq('Data Provider'))
            data_providers |= {rel.get('related_id') for rel in results}
        self.data_sources = list(data_providers)
        return self

    def load_asset_manager_permissions(self) -> 'AMaaSUser':
        if self.asset_manager_id is None:
            return self

        if not self.relationships:
            self.load_relationships()

        if not self.data_sources:
            self.load_data_sources()

        permissions = {0, self.asset_manager_id}
        permissions |= {rel.get('asset_manager_id') for rel in self.relationships}
        permissions |= {data_source for data_source in self.data_sources}
        self.asset_manager_permissions = list(permissions)

        return self

    def load_book_permissions(self) -> 'AMaaSUser':
        if self.asset_manager_id is None:
            return self

        results = self._dynamo_query('book_permissions',
                                     Select='SPECIFIC_ATTRIBUTES',
                                     IndexName='user_id_index',
                                     ProjectionExpression='asset_manager_id,book_id,#permission',
                                     KeyConditionExpression=Key('user_asset_manager_id').eq(self.asset_manager_id),
                                     ExpressionAttributeNames={"#permission": "permission"})

        self.book_permissions = results
        return self


    def check_asset_manager_permissions(self, asset_manager_id_claims: List[int]) -> (bool, List[int]):
        """
        Returns the asset_manager_ids that the user does not have access to
        """
        error_asset_manager_ids = [int(asset_manager_id) for asset_manager_id in asset_manager_id_claims
                                   if int(asset_manager_id) not in self.asset_manager_permissions]
        return error_asset_manager_ids


    def check_book_permissions(self, asset_manager_id: int, book_id: str, permission: str) -> None:
        if isinstance(asset_manager_id, str):
            asset_manager_id = int(asset_manager_id)

        if not self.book_permissions:
            self.load_book_permissions()

        if not self.relationships:
            self.load_relationships()

        # if user is an admin of the asset manager, he should have write permissions
        is_admin = next((rel for rel in self.relationships
                         if rel.get('asset_manager_id') == asset_manager_id
                         and rel.get('relationship_type') == 'Administrator'), None)

        if not is_admin:
            for perm in self.book_permissions:
                if (perm.get('book_id') == book_id or perm.get(book_id) == '*') and \
                    perm.get(asset_manager_id) == asset_manager_id:
                    if permission == 'write' and perm.get(permission) == 'read':
                        raise ValueError('User %s does not have requested %s access to the book: %s' \
                            % (self.asset_manager_id, permission, book_id))
                    return None
            raise ValueError('User %s does not have requested %s access to the book: %s' \
                    % (self.asset_manager_id, permission, book_id))


    def _dynamo_query(self, table, **kwargs):
        table_name = self.get_table_name(table)
        cache = self.__cache_clients[table_name]
        kwargs['TableName'] = table_name
        return cache.query(**kwargs).get('Items',[])



import os
from configparser import ConfigParser, NoSectionError

class AMaaSConfig(object):
    def __init__(self, config_path=None):
        if not config_path:
            config_path = os.path.join(os.path.expanduser('~'), '.amaas.cfg')
        if not os.path.exists(config_path):
            raise Exception('Could not read AMaaS config file: {}'.format(config_path))
        self.config = ConfigParser()
        self.config.read(config_path)
    
    def get_config_value(self, section, option):
        if not self.config.has_section(section):
            raise Exception('Missing [{}] in AMaaS config file'.format(section))
        if not self.config.has_option(section, option):
            raise Exception('Missing {} optin in [{}] section of AMaaS config file'.format(section, option))
        
        return self.config.get(section=section, option=option)


import boto3
from amaasinfra.infra.credential.policy import IoTPolicy

class STS(object):
    def __init__(self):
        self.client = boto3.client('sts')
    
    def assume_role(self):
        raise NotImplementedError()


class IoTSTS(STS):
    def __init__(self):
        super().__init__()

    def assume_role(self, role_arn, role_sesion_name, policy):
        return self.client.assume_role(RoleArn=role_arn,
                                       RoleSessionName=role_sesion_name,
                                       Policy=policy)






"""
Crude tests.  Need a better way to mock out the DynamoDB calls
"""
import unittest
from mock import patch
from amaasinfra.authorization.amaas_user import AMaaSUser
from amaasinfra.authorization.token_utils import TokenAttribute

class AMaaSUserTest(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass
    
    @patch('amaasinfra.authorization.amaas_user.unpack_token')
    @patch('amaasinfra.authorization.amaas_user.boto3')
    def testCheckAmidPermissions(self, mock_boto, mock_unpack_token):
        mock_unpack_token.return_value = {TokenAttribute.asset_manager_id.value: 1}
        user = AMaaSUser('')
        user.asset_manager_permissions = [1, 10, 11, 12, 101]

        errors = user.check_asset_manager_permissions([1, 10, 11, 12])
        self.assertTrue(len(errors) == 0)

        errors = user.check_asset_manager_permissions([101])
        self.assertTrue(len(errors) == 0)

        errors = user.check_asset_manager_permissions(['101'])
        self.assertTrue(len(errors) == 0)

        errors = user.check_asset_manager_permissions([101,102])
        self.assertTrue(102 in errors)

        with self.assertRaises(ValueError):
            errors = user.check_asset_manager_permissions(['Invalid101'])
        
    @patch('amaasinfra.authorization.amaas_user.unpack_token')
    @patch('amaasinfra.authorization.amaas_user.boto3')
    def testCheckNewUserPermissions(self, mock_boto, mock_unpack_token):
        mock_unpack_token.return_value = {TokenAttribute.username.value: 'new_user'}
        user = AMaaSUser('')
        
        claims = [10, 11, 12, 101]
        errors = user.check_asset_manager_permissions(claims)
        self.assertEqual(claims, errors)
    
    def testLoadRelationships(self):
        pass
    
    def testLoadDataSources(self):
        pass
    
    def testLoadAssetManagerIdPermissions(self):
        pass

if __name__ == '__main__':
    unittest.main()


from setuptools import setup, find_packages

setup(
    name='amaasinfra',
    keywords=['amaas', 'infra', 'aws'],
    description='This is an essential package for managing AMaaS infra layer',
    license='Apache License 2.0',
    install_requires=['aws-requests-auth',
                      'boto3',
                      'configparser',
                      'elasticsearch-dsl',
                      'pyjwt',
                      'requests',
                      'troposphere',
                      'pymysql'],
    version='0.2.63',
    author='AMaaS Pte Ltd',
    author_email='tech@argomi.com',
    packages=find_packages(),
    platforms='any',
)



"""
This module is used to manage transaction emails
"""
import boto3
import json
import requests
import pkg_resources
from amaasinfra.email.templates import EMAIL_TEMPLATES
from amaasinfra.infra.s3.s3_object import get_content, get_object

class EmailClient(object):
    """
    This is a template class that all email clients should inherite from
    """
    def __init__(self):
        pass

    def send_mail(self, **kwargs):
        """
        This method should be implemented by every email client
        """
        raise NotImplementedError()


class SESClient(EmailClient):
    """
    This is an implementation of SES
    """
    def __init__(self, region='us-west-2'):
        self.client = boto3.client('ses', region_name=region)

    def send_mail(self, email, *args, **kwargs):
        """
        Implementation of SES sending email
        """
        if isinstance(email, SimpleEmail):
            message = email.message

            if email.template and kwargs:
                s3_data = get_object(bucket='argomi.email.templates', key=email.template)
                message = get_content(s3_data)

                for token, value in kwargs.items():
                    message = message.replace('{%s}' % token, value)

            return self.client.send_email(Source=email.sender,
                                          Destination={'ToAddresses': [email.to]},
                                          Message={'Subject': {'Data': email.subject,
                                                               'Charset': 'utf8'},
                                                   'Body': {'Text': {'Data': email.message,
                                                                     'Charset': 'utf8'},
                                                            'Html': {'Data': message,
                                                                     'Charset': 'utf8'}}},
                                          ReplyToAddresses=[email.sender])
        else:
            raise TypeError('Invalid parameter type, SimpleEmail object required.')


class SimpleEmail(object):
    def __init__(self, subject, message, to, sender='noreply@argomi.com', template_name=None):
        self._sender= sender
        self._to = to
        self._subject = subject
        self._message = message
        
        if template_name not in EMAIL_TEMPLATES:
            raise ValueError('Unknown template_name {}'.format(template_name))
        self._template = template_name

    @property
    def sender(self):
        return self._sender

    @property
    def to(self):
        return self._to

    @property
    def subject(self):
        return self._subject

    @property
    def message(self):
        return self._message

    @property
    def template(self):
        return EMAIL_TEMPLATES[self._template]

if __name__ == '__main__':
    SESClient().send_mail(SimpleEmail(subject='test email', message='test message', to='cheng.chen@amaas.com'))
  


""" Logs calls to the AMaaS APIs """

from datetime import datetime

import boto3

class APICallLogger(object):
    """ The current implementation of the API Call Logger uses Cloudwatch metrics """

    def __init__(self):
        self.client = boto3.client('cloudwatch')

    def record_api_call(self, asset_manager_id, api_call_type, api_call_subtype, client_id=None, token_asset_manager_id=None, username=None, environment='dev'):
        """ Record the API call """
        asset_manager_id = str(asset_manager_id)  # In case it gets sent in as an integer
        dimentions = [
                        {'Name': 'api_call_type', 'Value': api_call_type},
                        {'Name': 'api_call_subtype', 'Value': api_call_subtype},
                        {'Name': 'asset_manager_id', 'Value': str(asset_manager_id)},
                        {'Name': 'environment', 'Value': environment},
                    ]

        if client_id:
            dimentions.append({'Name': 'client_id', 'Value': str(client_id)})

        if token_asset_manager_id:
            dimentions.append({'Name': 'token_asset_manager_id', 'Value': str(token_asset_manager_id)})

        if username:
            dimentions.append({'Name': 'username', 'Value': username})

        self.client.put_metric_data(
            Namespace='amaas',
            MetricData=[
                {
                    'MetricName': 'AMaaSAPICalls',
                    'Dimensions': dimentions,
                    'Timestamp': datetime.utcnow(),
                    'Value': 1.0,
                    'Unit': 'Count'
                },
            ]
        )

if __name__ == '__main__':
    APICallLogger().record_api_call(asset_manager_id=101,
                                    api_call_type='asset',
                                    api_call_subtype='amend',
                                    environment='dev')


from aws_requests_auth.boto_utils import BotoAWSRequestsAuth
from elasticsearch import Elasticsearch, RequestsHttpConnection
from elasticsearch.helpers import bulk
from elasticsearch_dsl import Search, Index

class ElasticsearchProxy(object):
    def __init__(self, endpoint, region, index):
        self.auth = BotoAWSRequestsAuth(aws_host=endpoint,
                                        aws_region=region,
                                        aws_service='es')
        self.client = Elasticsearch(host=endpoint,
                                    http_auth=self.auth,
                                    port=80,
                                    connection_class=RequestsHttpConnection)
        self.index = Index(index, using=self.client)
        self.index_name = index
        self.indexed_fields, self.text_fields = self.get_indexed_fields()

    def get_indexed_fields(self):
        indexed_fields = set()
        text_fields = set()
        mappings = self.index.get_mapping().get(self.index_name, {}).get('mappings', {}).values()
        for mapping in mappings:
            for prop, prop_value in mapping.get('properties', {}).items():
                child_properties = prop_value.get('properties')
                # flatten the nested field names
                if child_properties:
                    for child_prop, child_value in child_properties.items():
                        child_prop_name = '.'.join([prop, child_prop])
                        text_fields = text_fields if child_value.get('type') != 'text' \
                                      else text_fields | {child_prop_name}
                        indexed_fields.add(child_prop_name)
                else:
                    text_fields = text_fields if prop_value.get('type') != 'text'\
                                  else text_fields | {prop}
                    indexed_fields.add(prop)
        return indexed_fields, text_fields

    def search(self, asset_manager_ids, queries=[], filters={}, sort_fields=None,
               fuzzy=False, page_no=1, page_size=100, threshold=None):
        # force to have asset_manager_ids filter
        if not asset_manager_ids:
            raise ValueError('An asset manager id is required.')
        filters.pop('asset_manager_id', [])
        field_filters = [{field if field not in self.text_fields
                          else '{}.keyword'.format(field): filter_value}
                         for field, filter_value in filters.items()]

        for query_type in ['multi_match', 'query_string']:
            search = Search(using=self.client, index=self.index_name) \
                        .filter('terms', asset_manager_id=asset_manager_ids)
            for field_filter in field_filters:
                search = search.filter('terms', **field_filter)
            for query_arg, fields in queries:
                if query_arg:
                    search = search.query(query_type,
                                          query=query_arg if query_type != 'query_string' else '*{}*'.format(query_arg),
                                          fields=fields,
                                          fuzziness='AUTO' if fuzzy else 0)
            if sort_fields:
                sort_fields = ['{}.keyword'.format(field)
                               if field.replace('-', '') in self.text_fields else field
                               for field in sort_fields]
                search = search.sort(*sort_fields)

            result = search[page_no-1:page_size].execute()
            if result.hits.total:
                break
        response = {'total': result.hits.total,
                    'max_score': result.hits.max_score}
        items = []
        for hit in result.hits:
            if (threshold and result.hits.max_score and
                ((hit.meta.score / result.hits.max_score) * 100) < threshold):
                continue
            item = hit.to_dict()
            item.update({'_score': hit.meta.score})
            items.append(item)
        response.update({'hits': items})

        return response

    def bulk_update(self, documents):
        bulk(self.client, documents)


import os
from enum import Enum
from typing import Dict
import jwt

class TokenAttribute(Enum):
    first_name = 'given_name'
    last_name = 'family_name'
    email = 'email'
    username = 'cognito:username'
    asset_manager_id = 'custom:asset_manager_id'

def unpack_token(token: str) -> Dict[str, str]:
    """
    Unpacks the Cognito token attributes into a list of their values.
    
    Args:
        token: the encoded jwt token from the Authorization request header
        *additional_attributes: optional token attributes to be unpacked and returned
    
    Returns:
        Dict[TokenAttribute, str]: the list containing the attribute values
    """
    contents = jwt.decode(token, verify=False)
    return {attr.value: contents.get(attr.value, None) for attr in TokenAttribute}



import boto3
import json


class CloudManager(object):
    """
    This class is an abstract class for managing cloud infra
    """
    def __init__(self):
        pass

    def list_all_instance(self):
        """
        Abstract method to be implemented by derived classes
        """
        raise NotImplementedError()


class AWSManager(CloudManager):
    """
    This is a simple infra manager for AWS
    """
    def __init__(self):
       pass

    def list_all_api_gateways(self):
        client = boto3.client('apigateway')


